/*
 * mpl-udev.c
 *
 * by Alexander Zhirov (alexander@zhirov.kz)
 * Telegram @alexanderzhirov
 */

#include "mpl-lib.h"
#include "mpl-udev.h"
#include <libudev.h>

void mpl_udev_init(MPL_UDEV *mpl_udev)
{
    mpl_udev->udev = udev_new();
    if (mpl_udev->udev == NULL) {
        MPLOG(LOG_ERR, "Error creating udev context");
        exit(EXIT_FAILURE);
    }

    mpl_udev->monitor = udev_monitor_new_from_netlink(mpl_udev->udev, "udev");
    if (mpl_udev->monitor == NULL) {
        MPLOG(LOG_ERR, "Error creating udev monitor");
        udev_unref(mpl_udev->udev);
        exit(EXIT_FAILURE);
    }

    udev_monitor_filter_add_match_subsystem_devtype(mpl_udev->monitor,
                                                    "usb", NULL);
    udev_monitor_enable_receiving(mpl_udev->monitor);

    mpl_udev->udev_fd = udev_monitor_get_fd(mpl_udev->monitor);
    if (mpl_udev->udev_fd < 0) {
        MPLOG(LOG_ERR, "Error creating udev monitor descriptor");
        udev_unref(mpl_udev->udev);
        exit(EXIT_FAILURE);
    }
}

void mpl_udev_check_devices(MPL_UDEV *mpl_udev)
{
    struct udev_device *dev;
    struct udev_enumerate *enumerate;
    struct udev_list_entry *devices, *dev_list_entry;

    enumerate = udev_enumerate_new(mpl_udev->udev);

    if (enumerate == NULL) {
        MPLOG(LOG_ERR, "Error creating udev enumerate");
        exit(EXIT_FAILURE);
    }

    udev_enumerate_add_match_subsystem(enumerate, "usb");
    udev_enumerate_scan_devices(enumerate);

    devices = udev_enumerate_get_list_entry(enumerate);

    if (devices == NULL) {
        MPLOG(LOG_ERR, "Error get udev devices");
        exit(EXIT_FAILURE);
    }

    udev_list_entry_foreach(dev_list_entry, devices) {
        const char *syspath = udev_list_entry_get_name(dev_list_entry);
        dev = udev_device_new_from_syspath(mpl_udev->udev, syspath);

        const char *devtype = udev_device_get_devtype(dev);
        if (!(strcmp(devtype, "usb_interface") == 0))
            continue;
        
        const char *driver = udev_device_get_driver(dev);
        if (!(strcmp(driver, "option") == 0))
            continue;

        const char *sysattr_value = udev_device_get_sysattr_value(dev,
                                                    "bInterfaceNumber");
        printf("Номер порта: %s\n", sysattr_value);

        // int num = atoi(sysattr_value);

        const char *devsyspath = udev_device_get_syspath(dev);
        printf("syspath: %s\n", devsyspath);
        // device_path(syspaths, num);

        udev_device_unref(dev);
    }

    udev_enumerate_unref(enumerate);
}

mpl_udev_free(MPL_UDEV *mpl_udev)
{
    udev_monitor_unref(mpl_udev->monitor);
    udev_unref(mpl_udev->udev);
}
